#!/usr/bin/env python

'''
Check status description:

# General

## Check failed
ovs_bridge check=0

## Check succeeded
ovs_bridge check=1

# Interface, port, bridge

## Interface is down
ovs_bridge,interface=xxx,type=zzz,bridge=yyy,port=aaa status=0

## Interface is up
ovs_bridge,interface=xxx,type=zzz,bridge=yyy,port=aaa status=1

## Interface error
ovs_bridge,interface=xxx,type=zzz,bridge=yyy,port=aaa status=2

# Special errors tracking

## Bridge has no ports
ovs_bridge_port,bridge=yyy count=0

## Bridge port has no interface
ovs_bridge_port_iface,bridge=yyy,port=aaa count=0
'''

import sys

from ovsdbapp.backend.ovs_idl import connection
from ovsdbapp.schema.open_vswitch import impl_idl

SOCKET_PATH = 'unix:/run/openvswitch/db.sock'
DB_NAME = 'Open_vSwitch'
CONN_TIMEOUT = 5

CHK_TMPL = 'ovs_bridge check={}'
BR_CHK_TMPL = '{},{} status={}'
BR_CNT_TMPL = '{},{} count={}'


def format_labels(bridge, port=None, interface=None):
    label_obj = {
        'bridge': bridge.name
    }
    if port is not None:
        label_obj.update({
            'port': port.name
        })
    if interface is not None:
        label_obj.update({
            'interface': interface.name,
            'type': interface.type,
            'peer': interface.options.get('peer')
        })
    labels = ','.join(['{}={}'.format(key, value)
                       for key, value in label_obj.iteritems()
                       if not (value is None or len(value) == 0)])
    return labels


def gather():
    idl = connection.OvsdbIdl.from_server(SOCKET_PATH, DB_NAME)
    conn = connection.Connection(idl=idl, timeout=CONN_TIMEOUT)

    api = impl_idl.OvsdbIdl(conn)

    for br_id, bridge in api._tables['Bridge'].rows.items():

        print(BR_CNT_TMPL.format('ovs_bridge_port',
                                 format_labels(bridge),
                                 len(bridge.ports)))

        for port in bridge.ports:
            print(BR_CNT_TMPL.format('ovs_bridge_port_iface',
                                     format_labels(bridge, port),
                                     len(port.interfaces)))

            for iface in port.interfaces:
                labels = format_labels(bridge, port, iface)

                if len(iface.link_state) == 1 and iface.link_state[0] == 'up':
                    status = 1
                    if len(iface.error) > 0:
                        status = 2
                else:
                    status = 0

                print(BR_CHK_TMPL.format('ovs_bridge', labels, status))


if __name__ == "__main__":
    try:
        gather()
        print(CHK_TMPL.format(1))
    except Exception as e:
        print(CHK_TMPL.format(0))
        sys.stderr.write(str(e))
        sys.exit(1)
